<?php

declare(strict_types=1);

namespace Dedi\SyliusBlogPlugin\Entity;

use DateTimeInterface;
use Dedi\SyliusBlogPlugin\Domain\Blog\Adapter\AuthorInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\ImageInterface;
use Sylius\Component\Resource\Model\TimestampableTrait;
use Sylius\Component\Resource\Model\TranslationInterface;
use Symfony\Component\Validator\Constraints as Assert;

class Article extends AbstractReferable implements ArticleInterface
{
    use TimestampableTrait;

    protected ?int $id = null;

    protected bool $enabled = false;

    /** @Assert\PositiveOrZero(groups={"default", "sylius"}) */
    protected int $position = 0;

    protected ?ImageInterface $banner = null;

    protected Collection $categories;

    protected Collection $channels;

    protected ?AuthorInterface $author = null;

    protected Collection $comments;

    protected ?TypeArticleInterface $type = null;

    public function __construct()
    {
        parent::__construct();

        $this->categories = new ArrayCollection();
        $this->channels = new ArrayCollection();
        $this->comments = new ArrayCollection();
    }

    protected function createTranslation(): TranslationInterface
    {
        return new ArticleTranslation();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getPosition(): int
    {
        return $this->position;
    }

    public function setPosition(int $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function getBanner(): ?ImageInterface
    {
        return $this->banner;
    }

    public function setBanner(?ImageInterface $banner): self
    {
        $banner->setOwner($this);
        $this->banner = $banner;

        return $this;
    }

    public function getChannels(): Collection
    {
        return $this->channels;
    }

    public function setChannels(Collection $channels): self
    {
        $this->channels = $channels;

        return $this;
    }

    public function addChannel(ChannelInterface $channel): bool
    {
        if (!$this->channels->contains($channel)) {
            return $this->channels->add($channel);
        }

        return false;
    }

    public function removeChannel(ChannelInterface $channel): bool
    {
        return $this->channels->removeElement($channel);
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getSlug(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getSlug();
    }

    public function setSlug(string $slug): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setSlug($slug);

        return $this;
    }

    public function getTitle(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getTitle();
    }

    public function setTitle(string $title): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setTitle($title);

        return $this;
    }

    public function getContent(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getContent();
    }

    public function setContent(string $content): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setContent($content);

        return $this;
    }

    public function getCategories(): Collection
    {
        return $this->categories;
    }

    public function setCategories(Collection $categories): self
    {
        $this->categories = $categories;

        return $this;
    }

    public function hasCategories(): bool
    {
        return !$this->categories->isEmpty();
    }

    public function addCategory(CategoryInterface $category): bool
    {
        if (!$this->categories->contains($category)) {
            return $this->categories->add($category);
        }

        return false;
    }

    public function removeCategory(CategoryInterface $category): bool
    {
        return $this->categories->removeElement($category);
    }

    public function getAuthor(): ?AuthorInterface
    {
        return $this->author;
    }

    public function setAuthor(?AuthorInterface $author): self
    {
        $this->author = $author;

        return $this;
    }

    public function getCreatedAt(): ?DateTimeInterface
    {
        return $this->createdAt;
    }

    public function getEditedAt(): ?DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function getMetadataTitle(): ?string
    {
        return $this->getTranslation()->getMetadataTitle() ?? $this->getTitle();
    }

    public function getMetadataDescription(): ?string
    {
        return $this->getTranslation()->getMetadataDescription() ?? substr(strip_tags($this->getContent()), 0, 155);
    }

    public function getTranslation(?string $locale = null): AbstractReferableTranslation
    {
        /** @var ArticleTranslation $translation */
        $translation = parent::getTranslation($locale);

        return $translation;
    }

    public function getDescription(): ?string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getDescription();
    }

    public function setDescription(?string $description): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setDescription($description);

        return $this;
    }

    public function getComments(): Collection
    {
        return $this->comments;
    }

    public function addComment(CommentInterface $comment): self
    {
        $this->comments[] = $comment;

        return $this;
    }

    public function __toString(): string
    {
        return $this->getTitle();
    }

    public function getType(): ?TypeArticleInterface
    {
        return $this->type;
    }

    public function setType(?TypeArticleInterface $type): self
    {
        $this->type = $type;

        return $this;
    }
}
